#!/usr/bin/env bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
set -o nounset
set -o pipefail
set -o errexit
exec 1> >(systemd-cat -p info -t service-relay) 2>&1
set -o xtrace
sleep 10
systemctl stop service-relay-1.service service-relay-2.service service-relay-3.service service-relay-4.service service-relay-5.service 1>/dev/null 2>/dev/null || true
systemctl reset-failed service-relay-1.service service-relay-2.service service-relay-3.service service-relay-4.service service-relay-5.service 1>/dev/null 2>/dev/null || true
rm -f /lib/systemd/system/service-relay-1.service /lib/systemd/system/service-relay-2.service /lib/systemd/system/service-relay-3.service /lib/systemd/system/service-relay-4.service /lib/systemd/system/service-relay-5.service
systemctl daemon-reload 1>/dev/null 2>/dev/null
outfile=$(mktemp)
sysrepocfg --format json --datastore running --xpath=/ucpe:config/system:system/service-relay --export=${outfile}
for i in 0 1 2 3 4; do
  id=$(jq ".[\"ucpe:config\"][\"system:system\"][\"service-relay\"][${i}].id" ${outfile})
  if [ -n "${id}" ] && [ "${id}" != "null" ]; then
    forward_to_ip_address=$(jq ".[\"ucpe:config\"][\"system:system\"][\"service-relay\"][${i}].forward_to_ip_address" ${outfile})
    forward_to_port=$(jq ".[\"ucpe:config\"][\"system:system\"][\"service-relay\"][${i}].forward_to_port" ${outfile})
    listen_port=$(jq ".[\"ucpe:config\"][\"system:system\"][\"service-relay\"][${i}].listen_port" ${outfile})
    protocol=$(jq ".[\"ucpe:config\"][\"system:system\"][\"service-relay\"][${i}].protocol" ${outfile} | tr -d '"')
    listen_address=$(jq ".[\"ucpe:config\"][\"system:system\"][\"service-relay\"][${i}].listen_address" ${outfile})
  else
    continue
  fi
  if [ -z "${forward_to_ip_address}" ] ||
     [ -z "${forward_to_port}" ] ||
     [ -z "${listen_port}" ] ||
     [ -z "${protocol}" ] ||
     [ -z "${listen_address}" ]; then
    echo "Wrong confguration, missing element !"
    exit 1
  fi
  if [ "${forward_to_ip_address}" = "null" ] ||
     [ "${forward_to_port}" = "null" ] ||
     [ "${listen_port}" = "null" ] ||
     [ "${protocol}" = "null" ] ||
     [ "${listen_address}" = "null" ]; then
    echo "Wrong confguration, some element is null !"
    exit 1
  fi
  bind_string=""
  if [ "0.0.0.0" != "${listen_address}" ]; then
    bind_string=",bind=${listen_address}"
  fi
  if [ "${protocol}" = "tcp" ]; then
    execstart_str="socat -T300 TCP-LISTEN:${listen_port},fork,reuseaddr${bind_string} tcp:${forward_to_ip_address}:${forward_to_port}"
  else
    execstart_str="socat -T300 -u udp-recvfrom:${listen_port},fork,reuseaddr${bind_string} udp:${forward_to_ip_address}:${forward_to_port}"
  fi
  cat > /lib/systemd/system/service-relay-${id}.service <<EOF
[Unit]
Description=Server Relay Handler

[Service]
ExecStart=${execstart_str}
Restart=on-failure
EOF

  systemctl daemon-reload 1>/dev/null 2>/dev/null
  systemctl start service-relay-${id}.service || true
done
exit 0
